grammar Delta;

options {
 output = AST;
 ASTLabelType =CommonTree;
 backtrack = true;
}

tokens {
	STATEMENT_LIST;
	DECLARATIONS;
	DECLARATION;
	ASSIGNMENT;
	FUNCTION_CALL;
	FUNCTION_PARAMETER;	
	INT;
	FLOATING_POINT;
	STRING;
	ID;
	IF_STATEMENT;
	WHILE_STATEMENT;
}

// START:stat
prog:   declarations stat+ -> ^(STATEMENT_LIST declarations stat+);

block 	: '{'
		stat*
	  '}' -> ^(STATEMENT_LIST stat*)
	;

stat:   assignment ';' -> assignment
    | ifstat
    | whilestat
    | function_call ';' -> function_call
    | block 
    ;
    
 ifstat	:	'if' expr a=block ('else' b=block)? -> ^(IF_STATEMENT expr $a $b?)
    ;

whilestat
	:	 'while' expr block -> ^(WHILE_STATEMENT expr block)
	;

// END:stat

declarations
	:	
	declaration ';' (declaration ';')* -> ^(DECLARATIONS declaration*)
	;

declaration
	: vartype ID ( ',' ID)*   -> ^(DECLARATION vartype ID*);
		
vartype	:	
	'int'
	| 'string'
	| 'float'
	;

function_call   : 
	built_in_functions '(' function_parameter ')' -> ^(FUNCTION_CALL built_in_functions function_parameter)
	;
	
function_parameter
	:
	  expr (',' expr)* -> ^(FUNCTION_PARAMETER expr*)
	;	
	
built_in_functions
	:
	'print'
	| 'println'
	| 'input'
	| 'sin'
	| 'cos'
	;

assignment
	:			
	ID '=' expr -> ^(ASSIGNMENT ID expr)
	;
// START:expr

expr 	: condExpr;

condExpr:  aexpr (relationalOp^ aexpr)*
                ;

aexpr:   multExpr (('+'|'-')^ multExpr)*
    ; 

multExpr
    :   unaryExpr (('*'|'%'|'/')^ unaryExpr)*
    ; 


relationalOp 
    :    '<='
    |    '>='
    |   '<'
    |   '>'
    |    '=='
    |    '!='    
    ;
    
unaryExpr
	: unary_op^ atom
	| atom
	;    

atom:   INT 
    |	FLOATING_POINT
    |   '(' expr ')' -> expr
    | STRING
    | variable    
    | function_call
    ;

variable:
	ID
	;

unary_op:	
	PLUS
	|MINUS
	;   
// END:expr

// START:tokens
FLOATING_POINT
    :   ('0'..'9')+ '.' ('0'..'9')* Exponent?
    |   '.' ('0'..'9')+ Exponent? 
    |   ('0'..'9')+ ( Exponent)
    ;

fragment
Exponent : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

INT :   '0'..'9'+ ;
ID:    'a'..'z'+ ;
WS  :   (' '|'\t'|'\n'|'\r')+ {skip();} ;
STRING	:	 '\"' STRING_CONTENT '\"' ;

fragment
STRING_CONTENT :	
	( EscapeSequence | ~('\\'|'"') )* ;

fragment
EscapeSequence
    :   '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\')
    ;
    
PLUS	:	 '+';
MINUS :	 '-';

// END:tokens
