
class ValueObject {
	private DataType type;
	private String strValue;
	private double floatValue;
	private int intValue;

	public static ValueObject VOID_VALUE = new ValueObject();
	
	public String toString() {
		String s = type + " value = ";
		switch (type) {
		case INTEGER:
			s += (intValue);
			break;
		case FLOATING_POINT:
			s += (floatValue);
			break;
		case STRING:
			s += (strValue);
			break;
		case VOID:
			s += ("VOID");
			break;

		}
		return s;
	}

	public DataType getType() {
		return type;
	}

	public int getIntValue() {
		return intValue;
	}

	public double getFloatValue() {
		return floatValue;
	}

	public String getStringValue() {
		return strValue;
	}

	public ValueObject() {
		type = DataType.VOID;
	}
	
	public ValueObject( int ival) {		
		type = DataType.INTEGER;
		intValue = ival;
	}
	public ValueObject( double fval) {		
		type = DataType.FLOATING_POINT;
		floatValue = fval;
	}
	public ValueObject( String str) {		
		type = DataType.STRING;
		strValue = str;
	}

	/*cast from one object type to another*/
	ValueObject cast(DataType targettype) {
		/*the type is already correct*/
		if (type==targettype) {
			return this;
		}
		ValueObject result = null;
		/*cast integer to double*/
		if (targettype==DataType.FLOATING_POINT &&
		    type==DataType.INTEGER) {
			result = new ValueObject(intValue*1.0);
		}
		/*cast double to integer, use rounding*/
		if (targettype==DataType.INTEGER &&
		    type==DataType.FLOATING_POINT) {
			result = new ValueObject(Math.rint(floatValue));
		}
		return result;
	}

	ValueObject add(ValueObject other) throws InterpreterException {
		DataType result = TypeRule.getInstance().coerce("+", type,
					    other.type);
		ValueObject v1 = cast(result);
		ValueObject v2 = other.cast(result);
		switch (result) {
		case FLOATING_POINT:
			return new ValueObject(
				v1.floatValue + v2.floatValue);
		case INTEGER:
			return new ValueObject(
				v1.intValue + v2.intValue);
		case STRING:
			return new ValueObject(
				v1.strValue + v2.strValue);
		}
		return null;
	}

	ValueObject sub(ValueObject other) throws InterpreterException {
		DataType result = TypeRule.getInstance().coerce("-", type,
					    other.type);
		ValueObject v1 = cast(result);
		ValueObject v2 = other.cast(result);
		switch (result) {
		case FLOATING_POINT:
			return new ValueObject(
				floatValue - other.floatValue);
		case INTEGER:
			return new ValueObject(
				intValue - other.intValue);
		}
		return null;
	}


	ValueObject mul(ValueObject other) throws InterpreterException {
		DataType result = TypeRule.getInstance().coerce("*", type,
					    other.type);
		ValueObject v1 = cast(result);
		ValueObject v2 = other.cast(result);
		switch (result) {
		case FLOATING_POINT:
			return new ValueObject(
				v1.floatValue * v2.floatValue);
		case INTEGER:
			return new ValueObject(
				v1.intValue * v2.intValue);
		}
		return null;
	}

	ValueObject div(ValueObject other) throws InterpreterException {
		DataType result = TypeRule.getInstance().coerce("/", type,
					    other.type);
		ValueObject v1 = cast(result);
		ValueObject v2 = other.cast(result);
		switch (result) {
		case FLOATING_POINT:
			return new ValueObject(
				v1.floatValue / v2.floatValue);
		case INTEGER:
			return new ValueObject(
				v1.intValue / v2.intValue);
		}
		return null;
	}

	ValueObject mod(ValueObject other) throws InterpreterException {
		DataType result = TypeRule.getInstance().coerce("%", type,
					    other.type);
		ValueObject v1 = cast(result);
		ValueObject v2 = other.cast(result);
		switch (result) {
		case FLOATING_POINT:
			return new ValueObject(
				v1.floatValue % v2.floatValue);
		case INTEGER:
			return new ValueObject(
				v1.intValue % v2.intValue);
		}
		return null;
	}


	ValueObject neg() {
		switch (type) {
		case FLOATING_POINT:
			floatValue = -floatValue;
			break;
		case INTEGER:
			intValue = -intValue;
			break;
		}
		return this;
	}

	ValueObject compare(String operation, ValueObject other) throws InterpreterException{
		/* comparison is v1-v2 */
		DataType result = TypeRule.getInstance().coerce("-", type,
					    other.type);
		ValueObject v1 = cast(result);
		ValueObject v2 = other.cast(result);
		int compare_result = 0;

		switch (result) {
		case FLOATING_POINT:
			compare_result = Double.compare(v1.floatValue,
							v2.floatValue);
			break;
		case INTEGER:
			compare_result = v1.intValue - v2.intValue;
			break;
		case STRING:
			compare_result = v1.strValue.compareTo(v2.strValue);
		}
		boolean bresult = false;
		if (operation.equals("==")) 
			bresult = (compare_result == 0);
		if (operation.equals(">=")) 
			bresult = (compare_result >= 0);
		if (operation.equals("<=")) 
			bresult = (compare_result <= 0);
		if (operation.equals("<")) 
			bresult = (compare_result < 0);
		if (operation.equals(">")) 
			bresult = (compare_result > 0);
		return new ValueObject(bresult?1:0);
	}
}
